<?php
namespace Teknisa\MAN\Controllers;


use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\MAN\Services\ZeramentoSaldoService;
use Teknisa\MAN\Services;
use \Setforms2\Shared\ORM\DateTime;
use Teknisa\MAN\Services\EvoluiSaldo;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Services\NovoCodigo;
use Teknisa\MAN\Utilities;
use \Teknisa\Libs\Utilities\Environment;

class TabelaPrecoAlteracaoAutomatica extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $environment;
    
    public function __construct ($crudService, $parameterBag, Manager $dataSourceManager, Utilities\Util $utilService, $tabelaService, $environment){
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
        $this->utilService              = $utilService;
        $this->TabPrecAlterAutoServ     = $tabelaService;
        $this->environment              = $environment;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function findVigencias (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'vnd42400_tabelapreco_vigencias_operador';
        $this->parameterBagColumns = array('NRORG', 'CDOPERADOR');
        
        $filterCriteria = $request->getFilterCriteria();
        $rows = $this->utilService->getParamsAsArray($request);
        $conditions = $filterCriteria->getConditions();

        $operadorCondition = array_filter($conditions, function($item) {
            return strtoupper($item['columnName']) == 'CDOPERADOR';
        });
        
        if($operadorCondition == null) {
            $filterCriteria->addCondition("CDOPERADOR", "OPERADOR_LOGADO");
        }
        
        if($this->environment->getIdUtlPerfilFilial() == 'S'){
            $rows['NRPERFIL'] = $this->environment->getNrPerfil();
            $filterCriteria->addCondition("PERFIL", $rows['NRPERFIL']);
            
            // adicionando parametros
            $this->parameterBagColumns[] = "STATUS_VIGENCIA";
            $this->parameterBagColumns[] = "PERFIL";
            
            $this->dataSourceName = 'vnd42400_tabelapreco_vigencias_operador_perfil';
        }
        
        parent::find($request, $response);
    }
    
    public function findProdutos (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'vnd42400_tabelapreco_produto_alteracaoautomatica';
        $this->parameterBagColumns = array('NRORG', 'CDNVPRODUTO');
        parent::find($request, $response);
    }
    
    public function alterar(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $this->utilService->getParamsAsArray($request);
            $tabelas = $row['CDTABEPREC'];
            $qtdeTabelas = sizeof($tabelas);
            $i = 0;
            $alterarPrecoDiferenciado = false;
            $precosDiferenciados;
            $log = array();
            
            if (isset($row['NRDIASEMANPR']) && sizeof($row['NRDIASEMANPR']) > 0) {
                $alterarPrecoDiferenciado = true;
                
                if ($row['EXCLUIRPRECODIF'] == 'Y'){
                    $row['EXCLUIRPRECODIF'] = true;
                } else {
                    $row['EXCLUIRPRECODIF'] = false;
                }
            }
            
            if ($row['ALTERARPRECO'] == 'S') {
                $row['ALTERARPRECO'] = true;
            } else {
                $row['ALTERARPRECO'] = false;
            }
            
            $log[] = "Log de Alteração de Preço diferenciado";
            $log[] = "Data: ".date("d/m/Y H:i:s");
            $log[] = "";
            
            if($row['ALTERARPRECO']) {
                $log[] = "Opções selecionadas:";
                $log[] = "Inserir/Alterar Preço dos produtos que não possuem Preço cadastrado.";
            }
            
            if(isset($row['NRDIASEMANPR']) && sizeof($row['NRDIASEMANPR']) > 0 && $row['EXCLUIRPRECODIF']) {
                if(!$row['ALTERARPRECO']) {
                    $log[] = "Opção selecionada:";
                }
                $log[] = "Sobrescrever o preço dos produtos que já possuem preço diferenciado para o horário selecionado.";
                $log[] = "";
            }
            
            $qtdeProdutos = sizeof($row['CDPRODUTO']);
            
            for ($j = 0; $j < $qtdeProdutos; $j++) {
                $log[] = "Produto: ".$row['NMPRODUTO'][$j];
                
                
                $log[] = str_pad('-', 100, "-", STR_PAD_LEFT);
                
                for ($i = 0; $i < $qtdeTabelas; $i++) {
                    
                    $parametros = array(
                        'CDTABEPREC'    => $row['CDTABEPREC'][$i],
                        'GROUPED_LABEL' => $row['GROUPED_LABEL'][$i],
                        'DTINIVGPREC'   => $row['DTINIVGPREC'][$i],
                        'CDPRODUTO'     => $row['CDPRODUTO'][$j],
                        'CDFILIAL'      => $row['CDFILIAL'][$i],
                        'IDPRECVARIA'   => empty($row["IDPRECVARIA"]) ? 'N' : $row["IDPRECVARIA"],
                        'IDPRODPROMOC'  => empty($row["IDPRODPROMOC"]) ? 'N' : $row["IDPRODPROMOC"],
                        'VRPRECITEM'    => empty($row['VRPRECITEM']) ? 0 : $this->utilService->moneyToDouble($row["VRPRECITEM"]),
                        'CDPRPAITABPR'  => $row['CDPRODUTO'][$j],
                        'VRPRESUGITEM'  => 0,
                        'VRPRECITEMCL'  => empty($row['VRPRECITEMCL']) ? 0 : $this->utilService->moneyToDouble($row["VRPRECITEMCL"]),
                        'VRCUSTITEM'    => '',
                        'NRORG'         => $row['NRORG']
                    );
                    
                    $log[] = "Unidade: ".$parametros['CDFILIAL'].". Tabela: ".$parametros['GROUPED_LABEL'].". Início Vigência: ".$parametros['DTINIVGPREC'];
                    
                    $produtoExiste = $this->TabPrecAlterAutoServ->verificarProdutoExiste($parametros);
                    
                    if($produtoExiste) {
                        // pega o preço do produto para depois verificar se o desconto é maior
                        $precoProduto = $this->TabPrecAlterAutoServ->verificaPrecoDoProduto($parametros);
                        
                        // se for desconto maior que o valor, mostra erro na tela
                        if($row['IDDESCACREPR'] == 'D' && $row['IDPERVALORPR'] == 'V') {
                            $diferencaPrecoEDesconto = $precoProduto - $row['VRPRECODIA'];
                            
                            if($diferencaPrecoEDesconto < 0) {
                                $mensagem = "O valor informado de desconto deixa o produto com preço negativo";
                                
                                throw new \Exception($mensagem);
                            }
                        }
                    }
                    
                    if ($produtoExiste && $row['ALTERARPRECO']) {
                        $this->TabPrecAlterAutoServ->updateInformacoesVigencia($parametros);
                        array_push($log, "O produto teve seu preço alterado nessa tabela/vigência.");
                    } else if ($row['ALTERARPRECO']) {
                        $this->TabPrecAlterAutoServ->insertInformacoesVigencia($parametros);
                        $produtoExiste = true;
                        array_push($log, "O produto não existia nessa tabela/vigência. Foi inserido com os parâmetros informados.");
                    }
                    
                    if ($alterarPrecoDiferenciado && $produtoExiste) {
                        $arraySucesso = array();
                        foreach ($row['NRDIASEMANPR'] as $diaSemana) {
                            $arraySucesso[] = $this->inserirPrecosDiferenciados($parametros, $row, $diaSemana, $log);
                        } 
                    } else {
                        array_push($log, "Produto alterado com sucesso.");
                    }
                    
                    $log[] = str_pad('-', 100, "-", STR_PAD_LEFT);
                    $log[] = "";
                }
            }
            $response->addDataset(new DataSet('log', $log));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
     public function inserirPrecosDiferenciados($parametros, $row, $diaSemana, &$log){
        $podeSalvar = true;
        $excluiuProdutos = false;
        $sucesso = false;
        $excluirPrecoDiferenciado = $row['EXCLUIRPRECODIF'];
        $precoDiferenciado = $this->TabPrecAlterAutoServ->recuperarPrecoDiferenciado($parametros);
        
        $paramInsert = array_merge($parametros, array(
            ':VRPRECODIA'       => $row['VRPRECODIA'],
            ':DTFINVALPREC'     => $row['DTFINVALPREC'],
            ':IDPERVALORPR'     => $row['IDPERVALORPR'],
            ':IDDESCACREPR'     => $row['IDDESCACREPR'],
            ':HRFINPRECDIA'     => str_replace(':','',$row['HRFINPRECDIA']),
            ':IDVISUACUPOM'     => $row['IDVISUACUPOM'],
            ':NRDIASEMANPR'     => $diaSemana, 
            ':HRINIPRECDIA'     => str_replace(':','',$row['HRINIPRECDIA']),
            ':DTINIVALPREC'     => $row['DTINIVALPREC'],
            ':CDTIPOCONSPD'     => $row['CDTIPOCONS'], 
            ':NRORGULTATU'      => $row['NRORG']
        ));
        
        $diasDaSemana = array(
                '1' => "Domingo",
                '2' => "Segunda-feira",
                '3' => "Terça-feira",
                '4' => "Quarta-feira",
                '5' => "Quinta-feira",
                '6' => "Sexta-feira",
                '7' => "Sábado",
                'T' => "Todos",
                'F' => "Feriado",
                'V' => "Véspera de feriado");
        
        if(sizeof($precoDiferenciado) > 0) {
            $podeSalvar = true;
            
            foreach ($precoDiferenciado as $momento) {
                
                if(($paramInsert[':HRINIPRECDIA'] < $momento['HRFINPRECDIA'] ||
                   $paramInsert[':HRFINPRECDIA'] > $momento['HRINIPRECDIA']) &&
                   $diaSemana == $momento['NRDIASEMANPR'] && $row['CDTIPOCONS'] == $momento['CDTIPOCONSPD']) {
                    
                    if ($excluirPrecoDiferenciado) {
                        $paramDelete = array (
                            'CDFILIAL'      => $momento['CDFILIAL'],
                            'CDTABEPREC'    => $momento['CDTABEPREC'],
                            'DTINIVGPREC'   => $momento['DTINIVGPREC'],
                            'CDPRODUTO'     => $momento['CDPRODUTO'],
                            'CDPRPAITABPR'  => $momento['CDPRPAITABPR'],
                            'NRDIASEMANPR'  => $momento['NRDIASEMANPR'],
                            'HRINIPRECDIA'  => $momento['HRINIPRECDIA'],
                            'CDTIPOCONSPD'  => $momento['CDTIPOCONSPD'],
                            'NRORG'         => $momento['NRORG']
                        );
                        $this->TabPrecAlterAutoServ->apagarPrecoDiferenciado($paramDelete); 
                        $excluiuProdutos = true;
                    } else {
                        $podeSalvar = false;
                    }
                    
                }
            }
        }
        if ($podeSalvar) { 
            if($row['NMTIPOCONS'] == '') {
                $row['NMTIPOCONS'] = 'TODOS';
            }
            $this->TabPrecAlterAutoServ->insertPrecoDiferenciado($paramInsert);
            $log[] = "Dia da semana: ".$diasDaSemana[$diaSemana]." com Tipo Consumidor: ".$row['NMTIPOCONS']." incluído com sucesso";
        } else {
            $log[] = "Esse produto já possui um preço diferenciado no horário que deseja incluir. Dia da semana: ".$diasDaSemana[$diaSemana].". Tipo Consumidor: ".$row['NMTIPOCONS'];
        }
        if($excluirPrecoDiferenciado && $excluiuProdutos) 
            $log[] = "Foi necessário excluir preços diferenciados para esse produto. Dia da semana: ".$diasDaSemana[$diaSemana].". Tipo Consumidor: ".$row['NMTIPOCONS'];
        return $podeSalvar;
    }

    
}